@@ -1,52 +1,14 @@
-# $Id: Handel.pm 806 2005-09-12 03:21:54Z claco $
+# $Id: Handel.pm 1185 2006-05-31 02:04:55Z claco $
 package Handel;
 use strict;
 use warnings;
 use vars qw($VERSION);
+use Handel::ConfigReader;
 
-$VERSION = '0.18_01';
+$VERSION = '0.99_01';
 
-BEGIN {
-    use Handel::ConfigReader;
-
-    $Handel::Cfg = Handel::ConfigReader->new;
-
-    my $uuidsub;
-    if (eval{require UUID}) {
-        $uuidsub = sub {
-            my ($uuid, $uuidstring);
-            UUID::generate($uuid);
-            UUID::unparse($uuid, $uuidstring);
-
-            return $uuidstring;
-        };
-    } elsif (eval{require Data::UUID}) {
-        $uuidsub = sub {
-            my $ug = Data::UUID->new;
-            my $uuid = $ug->create;
-
-            return $ug->to_string($uuid);
-        };
-    } elsif (eval{
-            # for some reason 'no warnings' won't squelch
-            # the 'too late for INIT' warning in Win32::API::Type
-            local $^W = 0;
-            require Win32::Guidgen;
-        }) {
-        $uuidsub = sub {
-            return Win32::Guidgen::create();
-        };
-    } elsif (eval{require Win32API::GUID}) {
-        $uuidsub = sub {
-            return Win32API::GUID::CreateGuid();
-        };
-    } else {
-        throw Handel::Exception(
-            -text => 'Required modules not found',
-            -details => 'UUID/Data::UUID'
-        );
-    };
-    *Handel::newuuid = $uuidsub;
+sub config {
+    return Handel::ConfigReader->instance;
 };
 
 1;
@@ -54,7 +16,7 @@ __END__
 
 =head1 NAME
 
-Handel - Simple commerce framework with AxKit/TT2 support
+Handel - Simple commerce framework with AxKit/TT/Catalyst support
 
 =head1 DESCRIPTION
 
@@ -65,37 +27,47 @@ might be useful to others so here it is on CPAN.
 
 For the curious, Handel is German for commerce.
 
+=head1 METHODS
+
+=head2 config
+
+Returns a Handel::ConfigReader instance.
+
 =head1 WEBSITE
 
-You can get the latest news, source, documentation and wiki help at http://handelframework.com/.
+You can get the latest news, source, documentation and wiki help at
+http://handelframework.com/.
 
 =head1 MAILING LIST
 
-Join the mailing list at http://lists.rawmode.org/cgi-bin/mailman/listinfo/handel.
-Big thanks to gabb@#catalyst for the list space.
+Join the mailing list at
+http://lists.rawmode.org/cgi-bin/mailman/listinfo/handel. Big thanks to
+gabb@#catalyst for the list space.
 
 =head1 GOALS
 
 =over
 
-=item Database agnostic. Thanks to Class::DBI and staying away from auto incrementing ids,
-Handel should run in any database that Class::DBI supports.
+=item Database agnostic. Thanks to DBIx::Class and staying away from auto
+incrementing ids, Handel should run in any database that DBIx::Class supports.
 
-=item Implementation agnostic. Handel should be able to be used from the command line,
-from a web page, from a SOAP service, or from a GUI application. It's simply a data access mechanism.
-Any interaction with forms, web pages, browser, cookies, etc is the responsibility of the consumer
+=item Implementation agnostic. Handel should be able to be used from the command
+line, from a web page, from a SOAP service, or from a GUI application. It's
+simply a data access mechanism. Any interaction with forms, web pages, browser,
+cookies, etc is the responsibility of the consumer
 
-=item Checkout agnostic. A checkout process means different things to different people in
-different situations. The Handel checkout pipeline by itself does absolutely nothing with
-an order. It is instead a plugin manager that allows you to specifiy and build plugins to do
-various actions (payment autorization, address validation, fax delivery, etc). Each site, server, page,
-even process() call can have it's own uniquie order processing pipeline.
+=item Checkout agnostic. A checkout process means different things to different
+people in different situations. The Handel checkout pipeline by itself does
+absolutely nothing with an order. It is instead a plugin manager that allows
+you to specify and build plugins to do various actions (payment authorization,
+address validation, fax delivery, etc). Each site, server, page, even process()
+call can have it's own unique order processing pipeline.
 
 =item Easy integration into AxKit using taglibs.
 
 =item Easy integration into TT2 using plugins.
 
-=item Easy intrgration into Catalyst using plugins.
+=item Easy integration into Catalyst using plugins.
 
 =item Easy integration into other systems using SOAP.
 
@@ -133,8 +105,9 @@ even process() call can have it's own uniquie order processing pipeline.
 Handel is a simple framework to load/upload cart/order data and shove that
 data through a plugin based pipeline.
 
-=item B<(1)> While it is not a complete web based commerce system, I do plan on having
-feature complete working demos sites so people can still quick-start their own projects.
+=item B<(1)> While it is not a complete web based commerce system, I do plan on
+having feature complete working demos sites so people can still quick-start
+their own projects.
 
 =back
 
@@ -163,8 +136,8 @@ feature complete working demos sites so people can still quick-start their own p
 =head2 Prerequisites
 
 The following modules are required for Handel to work properly. Older versions
-may work fine, but hese are the versions I have installed and verified to
-work correctly. IF you have older versious and all tests pass, send me an email
+may work fine, but these are the versions I have installed and verified to
+work correctly. If you have older versions and all tests pass, send me an email
 and I'll lower the version requirements.
 
 =over
@@ -251,7 +224,7 @@ C<Test::More> version 0.48 or greater.
 The C<Test::More> included with perl 5.8.4 and C<Test::More> <= 0.48 have issues
 with ithreads that usually cause crashes in tests that use C<Class::DBI> or
 C<DBIx:ContextualFetch>. The errors usual mention
-"attempt to free unreferenced scalar". If you reveive these during C<make test>,
+"attempt to free unreferenced scalar". If you receive these during C<make test>,
 try upgrading C<Test::More>.
 
 =item Pod::Coverage
@@ -285,6 +258,16 @@ This keeps me honest and makes sure I always C<use strict>.
 
 =back
 
+=head1 CAVEATS
+
+When using item_class to specify the item class returned from add/items in your
+subclass under Class::DBI < 3.0.8, the item_class specified will be returned
+from all cart classes, including Handel::Cart itself. In most cases, people are
+only using one subclass of Handel::Cart so this won't effect them.
+
+If you plan on using multiple subclasses of Handel::Cart that may or may not
+return custom items, upgrade your Class::DBI to version 3.0.8 or greater.
+
 =head1 SEE ALSO
 
 L<Handel::Cart>
