@@ -1,16 +1,23 @@
-# $Id: Checkout.pm 712 2005-08-11 01:27:43Z claco $
+# $Id: Checkout.pm 1180 2006-05-31 02:01:17Z claco $
 package Handel::Checkout;
 use strict;
 use warnings;
 
 BEGIN {
     use Handel;
+    use Handel::Cart;
+    use Handel::Checkout::Stash;
     use Handel::Constants qw(:checkout :returnas);
     use Handel::Constraints qw(constraint_checkout_phase constraint_uuid);
     use Handel::Exception qw(:try);
     use Handel::Checkout::Message;
     use Handel::L10N qw(translate);
+    use Handel::Order;
     use Module::Pluggable 2.95 instantiate => 'new', sub_name => '_plugins';
+    use base qw(Class::Data::Inheritable);
+
+    __PACKAGE__->mk_classdata(_order_class => 'Handel::Order');
+    __PACKAGE__->mk_classdata(_stash_class => 'Handel::Checkout::Stash');
 };
 
 sub new {
@@ -21,7 +28,7 @@ sub new {
         handlers => {},
         phases => [],
         messages => [],
-        stash => {}
+        stash => $opts->{'stash'} || $class->stash_class->new
     }, ref $class || $class;
 
     $self->_set_search_path($opts);
@@ -43,11 +50,11 @@ sub new {
 sub plugins {
     my $self = shift;
 
-    return wantarray ? @{$self->{'plugins'}} : $self->{'plugins'};
+    return wantarray ? sort @{$self->{'plugins'}} : $self->{'plugins'};
 };
 
 sub add_handler {
-    my ($self, $phase, $ref) = @_;
+    my ($self, $phase, $ref, $preference) = @_;
     my ($package) = caller;
 
     throw Handel::Exception::Argument( -details =>
@@ -62,7 +69,18 @@ sub add_handler {
 
     foreach (@{$self->{'plugins'}}) {
         if (ref $_ eq $package) {
-            push @{$self->{'handlers'}->{$phase}}, [$_, $ref];
+            if ($preference) {
+                if (exists $self->{'handlers'}->{$phase}->{$preference}) {
+                    my $plugin = $self->{'handlers'}->{$phase}->{$preference}->[0];
+
+                    throw Handel::Exception::Checkout( -details =>
+                        translate("There is already a handler in phase ([_1]) for preference ([_2]) from the plugin ([_3])", $phase, $preference, $plugin) . '.')
+                };
+            } else {
+                my @prefs = sort {$a <=> $b} keys %{$self->{'handlers'}->{'$phase'}};
+                $preference = scalar @prefs ? $#prefs++ : 101;
+            };
+            $self->{'handlers'}->{$phase}->{$preference} = [$_, $ref];
             last;
         };
     };
@@ -78,7 +96,7 @@ sub add_message {
         $message->line($line) unless $message->line;
 
         push @{$self->{'messages'}}, $message;
-    } elsif (!ref $message) {
+    } elsif (!ref $message || ref $message eq 'Apache::AxKit::Exception::Error') {
         push @{$self->{'messages'}}, Handel::Checkout::Message->new(
             text => $message, source => $package, filename => $filename, line => $line);
     } else {
@@ -87,12 +105,48 @@ sub add_message {
     };
 };
 
+sub add_phase {
+    my ($self, $name, $value, $import) = @_;
+    my $caller = (caller);
+
+    if (Handel::Constants->can($name)) {
+        throw Handel::Exception::Constraint(
+            -text => translate("A constant named '$name' already exists in Handel::Constants")
+        );
+    } elsif (constraint_checkout_phase($value)) {
+        throw Handel::Exception::Constraint(
+            -text => translate("A phase constant value of '$value' already exists")
+        );
+    } elsif ($import && main->can($name)) {
+        throw Handel::Exception::Constraint(
+            -text => translate("A constant named '$name' already exists in caller '$caller'")
+        );
+    } else {
+        my $sub = sub {return $value};
+        no strict 'refs';
+
+        *{"Handel::Constants::$name"} = $sub;
+
+        if ($import) {
+            *{"${caller}::$name"} = $sub;
+        };
+
+        push @Handel::Constants::CHECKOUT_ALL_PHASES, $value;
+    };
+};
+
+sub clear_messages {
+    my $self = shift;
+
+    $self->{'messages'} = [];
+};
+
 sub cart {
     my ($self, $cart) = @_;
 
     if ($cart) {
         if (ref $cart eq 'HASH' || UNIVERSAL::isa($cart, 'Handel::Cart') || constraint_uuid($cart)) {
-            $self->order(Handel::Order->new({cart => $cart}));
+            $self->order($self->order_class->new({cart => $cart}));
         } else {
             throw Handel::Exception::Argument( -details =>
                 translate('Param 1 is not a HASH reference, Handel::Cart object, or cart id') . '.');
@@ -106,16 +160,27 @@ sub messages {
     return wantarray ? @{$self->{'messages'}} : $self->{'messages'};
 };
 
+sub order_class {
+    my ($self, $order_class) = @_;
+
+    if ($order_class) {
+        eval "require $order_class";
+        $self->_order_class($order_class);
+    };
+
+    return $self->_order_class;
+};
+
 sub order {
     my ($self, $order) = @_;
 
     if ($order) {
         if (ref $order eq 'HASH') {
-            $self->{'order'} = Handel::Order->load($order, RETURNAS_ITERATOR)->first;
+            $self->{'order'} = $self->order_class->load($order, RETURNAS_ITERATOR)->first;
         } elsif (UNIVERSAL::isa($order, 'Handel::Order')) {
             $self->{'order'} = $order;
         } elsif (constraint_uuid($order)) {
-            $self->{'order'} = Handel::Order->load({id => $order});
+            $self->{'order'} = $self->order_class->load({id => $order})->first;
         } else {
             throw Handel::Exception::Argument( -details =>
                 translate('Param 1 is not a HASH reference, Handel::Order object, or order id') . '.');
@@ -130,16 +195,19 @@ sub phases {
 
     if ($phases) {
         throw Handel::Exception::Argument( -details =>
-            translate(
-                'Param 1 is not an ARRAY reference') . '.')
-                unless ref($phases) eq 'ARRAY';
+            translate('Param 1 is not an ARRAY reference or string') . '.') unless (ref($phases) eq 'ARRAY' || !ref($phases));
+
+        if (! ref $phases) {
+            # holy crap, that actually worked!
+            $phases = [map(eval "$_", _path_to_array($phases))];
+        };
 
         $self->{'phases'} = $phases;
     } else {
         if (wantarray) {
-            return @{$self->{'phases'}} || @{&CHECKOUT_DEFAULT_PHASES};
+            return (scalar @{$self->{'phases'}}) ? @{$self->{'phases'}} : @{&CHECKOUT_DEFAULT_PHASES};
         } else {
-            return scalar @{$self->{'phases'}} ? $self->{'phases'} : &CHECKOUT_DEFAULT_PHASES;
+            return (scalar @{$self->{'phases'}}) ? $self->{'phases'} : &CHECKOUT_DEFAULT_PHASES;
         };
     };
 };
@@ -151,8 +219,13 @@ sub process {
     if ($phases) {
         throw Handel::Exception::Argument( -details =>
             translate(
-                'Param 1 is not an ARRAY reference') . '.')
-                unless ref($phases) eq 'ARRAY';
+                'Param 1 is not an ARRAY reference or string') . '.')
+                unless (ref($phases) eq 'ARRAY' || ! ref($phases));
+
+        if (! ref $phases) {
+            # holy crap, that actually worked!
+            $phases = [map(eval "$_", _path_to_array($phases))];
+        };
     } else {
         $phases = $self->{'phases'} || CHECKOUT_DEFAULT_PHASES;
     };
@@ -161,21 +234,29 @@ sub process {
         translate('No order is assocated with this checkout process') . '.')
             unless $self->order;
 
+    $self->stash->clear;
     $self->_setup($self);
 
     {
-        local $self->order->db_Main->{AutoCommit};
-
-        %{$self->{'stash'}} = ();
+        $self->order->schema_instance->txn_begin;
 
         foreach my $phase (@{$phases}) {
-            foreach my $handler (@{$self->{'handlers'}->{$phase}}) {
+            next unless $phase;
+
+            my @handlerprefs = sort {$a <=> $b} keys %{$self->{'handlers'}->{$phase}};
+            foreach my $handlerpref (@handlerprefs) {
+                my $handler = $self->{'handlers'}->{$phase}->{$handlerpref};
                 my $status = $handler->[1]->($handler->[0], $self);
 
                 if ($status != CHECKOUT_HANDLER_OK && $status != CHECKOUT_HANDLER_DECLINE) {
                     $self->_teardown($self);
 
-                    eval {$self->order->dbi_rollback};
+                    $self->order->schema_instance->txn_rollback;
+                    $self->order->storage->discard_changes;
+                    foreach my $item ($self->order->items) {
+                        $item->storage->discard_changes;
+                    };
+
                     if ($@) {
                         throw Handel::Exception(-details => "Transaction aborted. Rollback failed: $@");
                     };
@@ -185,7 +266,7 @@ sub process {
             };
         };
 
-        $self->order->dbi_commit;
+        $self->order->schema_instance->txn_commit;
     };
 
     $self->_teardown($self);
@@ -193,6 +274,17 @@ sub process {
     return CHECKOUT_STATUS_OK;
 };
 
+sub stash_class {
+    my ($self, $stash_class) = @_;
+
+    if ($stash_class) {
+        eval "require $stash_class";
+        $self->_stash_class($stash_class);
+    };
+
+    return $self->_stash_class;
+};
+
 sub stash {
     my $self = shift;
 
@@ -225,7 +317,7 @@ sub _teardown {
 
 sub _set_search_path {
     my ($self, $opts) = @_;
-    my $config = $Handel::Cfg;
+    my $config = Handel->config;
 
     my $pluginpaths = ref $opts->{'pluginpaths'} eq 'ARRAY' ?
         join(' ', @{$opts->{'pluginpaths'}}) : $opts->{'pluginpaths'} || '';
@@ -278,7 +370,7 @@ __END__
 
 =head1 NAME
 
-Handel::Checkout - Checkout Pipeline Process
+Handel::Checkout - Checkout Pipeline Processor
 
 =head1 SYNOPSIS
 
@@ -336,7 +428,7 @@ to be passed.
 
 =item pluginpaths
 
-An array reference or a comma (or space) seperated list containing the various
+An array reference or a comma (or space) separated list containing the various
 namespaces of plugins to be loaded. This will override any settings in
 C<ENV> or F<httpd.conf> for the current checkout instance only.
 
@@ -353,7 +445,7 @@ plugin search paths.
 
 =item addpluginpaths
 
-An array reference or a comma (or space) seperated list containing the various
+An array reference or a comma (or space) separated list containing the various
 namespaces of plugin paths in addition to Handel::Checkout::Plugin to be loaded.
 If C<HandelAddPluginPaths> is also specified, the two will be combined.
 
@@ -370,7 +462,7 @@ plugin search paths.
 
 =item loadplugins
 
-An array reference or a comma (or space) seperated list containing the
+An array reference or a comma (or space) separated list containing the
 names of the specific plugins to load in the current plugin paths.
 
 See L<"HandelLoadPlugins"> for more information about loading specific
@@ -378,7 +470,7 @@ plugins.
 
 =item ignoreplugins
 
-An array reference or a comma (or space) seperated list containing the
+An array reference or a comma (or space) separated list containing the
 names of the specific plugins to be ignored (not loaded)
 in the current plugin paths.
 
@@ -387,32 +479,53 @@ plugins.
 
 =item phases
 
-An array reference containing the various phases to be executed.
+An array reference or a comma (or space) separated list containing the
+various phases to be executed.
 
     my $checkout = Handel::Checkout->new({
         phases => [CHECKOUT_PHASE_VALIDATION,
                    CHECKOUT_PHASE_AUTHORIZATION]
     });
 
+    my $checkout = Handel::Checkout->new({
+        phases => 'CHECKOUT_PHASE_VALIDATION, CHECKOUT_PHASE_AUTHORIZATION'
+    });
+
+=item stash
+
+A Handel::Checkout::Stash instance of subclass instance. If nothing is specified,
+$self->stash_class will be used instead.
+
 =back
 
 =head1 METHODS
 
-=head2 add_handler($phase, \&coderef)
+=head2 add_handler($phase, \&coderef, $preference)
 
-Registers a code reference with the checkout phase specified. This is
-usually called within C<register> on the current checkout context:
+Registers a code reference with the checkout phase specified and assigned a run
+order preference. This is usually called within C<register> on the current
+checkout context:
 
     sub register {
         my ($self, $ctx) = @_;
 
-        $ctx->add_handler(CHECKOUT_PHASE_DELIVER, \&myhandler);
+        $ctx->add_handler(CHECKOUT_PHASE_DELIVER, \&myhandler, 200);
     };
 
     sub myhandler {
         ...
     };
 
+If no preference number is specified, the handler is added to the end of the
+list after all other handlers in that phase.
+
+If there is already a handler in the specified phase with the same preference, a
+Handel::Exception::Checkout exception will be thrown.
+
+While not enforced, please keep your handler preference orders between 251 - 749.
+Preference orders 1-250 and 750-1000 will be reserved for core modules that need
+to run before or after all other plugin handlers.
+
 =head2 add_message($message)
 
 Adds a new text message or Handel::Checkout::Message based object
@@ -431,10 +544,31 @@ If your adding a simple text message, a new Handel::Checkout::Message object
 will automatically be created and C<package>, C<filename>, and C<line>
 properties will be set.
 
+=head2 add_phase($name, $value [, $import]);
+
+Adds a new constant/sub named $name to Handel::Constant and adds the $value to
+CHECKOUT_ALL_PHASES. The new phase will be accepted by &constraint_checkout_phase
+and can be used by checkout plugins registering their handlers via
+add_handler($phase, &handler). If $import is true, add_phase will register the
+constant in the local namespace just as if it you had specified it in your use
+statement.
+
+    use Handel::Checkout;
+
+    Handel::Checkout->add_phase('CHECKOUT_PHASE_CUSTOMPHASE', 42, 1);
+
+    print constraint_checkout_phase(&CHECKOUT_PHASE_CUSTOMPHASE);
+
+    $plugincontext->add_handler(Handel::Constants::CHECKOUT_PHASE_CUSTOMPHASE, &handlersub);
+
+=head2 clear_messages
+
+Clears all messages from the current checkout instance.
+
 =head2 cart
 
 Creates a new Handel::Order object from the specified cart and associates
-that order with the current checkout process. This is typeically only needed
+that order with the current checkout process. This is typically only needed
 the first time you want to run checkout for a specific cart. From then on,
 you only need to load the already created order using C<order> below.
 
@@ -455,9 +589,9 @@ the first one will be used.
 
 =item cart(Handel::Cart)
 
-You can also pass in an already existing Handel::Cart object. It will then
-be loaded into a new order object ans associated with the current checkout
-process.
+You can also pass in an already existing Handel::Cart object or subclass. It
+will then be loaded into a new order object ans associated with the current
+checkout process.
 
     my $cart = Handel::Cart->load({
         id => '12345678-9098-7654-3212-345678909876'
@@ -475,6 +609,34 @@ current checkout process.
 
 =back
 
+=head2 order_class($orderclass)
+
+Gets/Sets the name of the class to use when loading existing order into the
+checkout process. By default, it loads order using Handel::Order. While you can
+set this directly in your application, it's best to set it in a custom subclass
+of Handel::Checkout.
+
+    package CustomCheckout;
+    use base 'Handel::Checkout';
+    __PACKAGE__->order_class('CustomOrder');
+
+    ...
+    use CustomCheckout;
+    my $checkout = CustomCheckout->new({order => '11111111-2222-3333-4444-555555555555'});
+
+    print ref $checkout->order; # CustomOrder
+
+=head2 stash_class($stashclass)
+
+Gets/Sets the name of the stash class to create during C<new>. By default, it
+returns Handel::Checkout::Stash. While you can
+set this directly in your application, it's best to set it in a custom subclass
+of Handel::Checkout.
+
+    package CustomCheckout;
+    use base 'Handel::Checkout';
+    __PACKAGE__->stash_class('MyCustomStash');
+
 =head2 messages
 
 Returns a reference to an array in list context of Handel::Checkout::Message
@@ -519,8 +681,8 @@ the first one will be used.
 
 =item order(Handel::Order)
 
-You can also pass in an already existing Handel::Order object. It will then
-be associated with the current checkout process.
+You can also pass in an already existing Handel::Order object or subclass. It
+will then be associated with the current checkout process.
 
     my $order = Handel::Order->load({
         id => '12345678-9098-7654-3212-345678909876'
@@ -539,13 +701,16 @@ will be loaded and associated with the current checkout process.
 
 =head2 phases(\@phases)
 
-Get/Set the phases active for the current checkout process.
+Get/Set the phases active for the current checkout process. This can be
+an array reference or a comma (or space) separated string:
 
     $checkout->phases([
         CHECKOUT_PHASE_INITIALIZE,
         CHECKOUT_PHASE_VALIDATION
     ]);
 
+    $checkout->phases('CHECKOUT_PHASE_INITIALIZE, CHECKOUT_PHASE_VALIDATION']);
+
 No attempt is made to sanitize the array for duplicates or the order of the phases.
 This means you can do evil things like run a phase twice, or run the phases
 out of order. Returns a list in list context and an array reference in scalar context.
@@ -554,7 +719,15 @@ out of order. Returns a list in list context and an array reference in scalar co
 
 Executes the current checkout process pipeline and returns CHECKOUT_STATUS_*.
 Any plugin handler that doesn't return CHECKOUT_HANDLER_OK or CHECKOUT_HANDLER_DECLINE
-is considered to be an error that the chekcout process is aborted.
+is considered to be an error that the checkout process is aborted.
+
+Just like C<phases>, you can pass an array reference or a comma (or space)
+separated string of phases into process.
+
+The method $self->stash->clear is called before the call to
+$plugin->setup so plugins can set stash data, and the stash remains until the
+next call to process so $plugin->teardown can read any remaining stash data
+before C<process> ends.
 
 The call to C<process> will return on of the following constants:
 
@@ -573,7 +746,8 @@ the registered plugin handlers.
 
 =head2 stash
 
-Returns a hash collection of information shared by all plugins in the current context.
+Returns a Handel::Checkout::Stash object that can store information shared by
+all plugins in the current context.
 
     # plugin handler
     my ($self, $ctx) = @_;
@@ -593,7 +767,7 @@ In the example above, the checkout plugin search path will load all plugins
 in the MyApp::Plugins::* namespace (but not MyApp::Plugin itself). Any plugins
 in Handel::Checkout::Plugin::* will be ignored.
 
-You can also pass a comma or space seperate list of namespaces.
+You can also pass a comma or space separate list of namespaces.
 
     PerlSetVar HandelPluginPaths 'MyApp::Plugins, OtherApp::Plugins'
 
@@ -603,7 +777,7 @@ will be ignored.
 =head2 HandelAddPluginPaths
 
 This adds an additional plugin search paths. This can be a comma or space
-seperated list of namespaces.
+separated list of namespaces.
 
     PerlSetVar HandelAddPluginPaths  'MyApp::Plugins, OtherApp::Plugins'
 
@@ -616,7 +790,7 @@ will be ignored.
 
 =head2 HandelIgnorePlugins
 
-This is a comma/space seperated list [or an anonymous array, or a regex outside of httpd.conf] of plugins to ignore when loading
+This is a comma/space separated list [or an anonymous array, or a regex outside of httpd.conf] of plugins to ignore when loading
 all available plugins in the given namespaces.
 
     PerlSetVar HandelIgnorePlugins 'Handel::Checkout::Plugin::Initialize'
@@ -641,7 +815,7 @@ they will be ignored.
 
 =head2 HandelLoadPlugins
 
-This is a comma or space seperated list [or an anonymous array, or a regex outside of httpd.conf] of plugins to be loaded from the available namespaces.
+This is a comma or space separated list [or an anonymous array, or a regex outside of httpd.conf] of plugins to be loaded from the available namespaces.
 
     PerlSetVar HandelLoadPlugins 'Handel::Checkout::Plugin::ValidateAddress'
 
@@ -667,7 +841,7 @@ they will be ignored.
 
 =head1 CAVEATS
 
-[I think] Due to the localization of AutoCommit to coeerse disabling of autoupdates during process,
+[I think] Due to the localization of AutoCommit to coerce disabling of autoupdates during process,
 Always access orders and order items from their checkout parent once they've been assigned to
 the checkout process, and not any available reference:
 
